/**
 * Header-fil for rute-klassen
 *
 * @file rute.h
 * @author Sondre Sand & Frederik Simonsen
 */

#ifndef __RUTE_H
#define __RUTE_H

#include <list>
#include <string>
#include <fstream>
#include "enum.h"

/**
 * Stopp (med unikt nummer og antall minutter fra stoppested)
 */
struct Stopp {
    int nr, minutter;
    Stopp(const int stoppNr, const int antMin);
    int hentMin();
    int hentNr();
    void skrivTilFil(std::ofstream & ut);

};

/**
 * Rute (med oversikt over alle stoppestedene på rutene)
 */
class Rute {
    private:
        std::list <Stopp*> stoppene;

    public:
        Rute();
        Rute(std::ifstream & inn);
        bool erListeGyldig();
        bool Rute::erNabo(Stopp* s1, Stopp* s2);
        bool erTall(std::string nvn);
        bool finnesStopp(const int nr);
        bool gyldigTid(const int time, const int minutt);
        int finnDifferanse(const int nr, const Retning retning);
        Stopp* finnStopp(int nr);
        int hentRuteIndeks(const int nr);
        std::string hentNavn(const Retning retning);
        void fjernStopp();
        void ruteTabell(const int diff, const std::string start,
                        const std::string stSted);
        void skrivRetninger() const;
        void skrivRute(const Retning retning);
        void slettData();
        void smettStopp();
        virtual void lesData();
        virtual void skrivBeskrivelse();
        virtual void skrivData() const;
        virtual void skrivKort() const;
        virtual void skrivTilFil(std::ofstream & ut);


};


#endif
